@@ -71,15 +71,12 @@ enum {
   PROP_EDITOR_MODE
 };
 
-gchar *icon_names[YELP_SETTINGS_NUM_ICONS];
+static const gchar *icon_names[YELP_SETTINGS_NUM_ICONS];
 
-G_DEFINE_TYPE (YelpSettings, yelp_settings, G_TYPE_OBJECT);
+G_DEFINE_TYPE (YelpSettings, yelp_settings, G_TYPE_OBJECT)
 #define GET_PRIV(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_SETTINGS, YelpSettingsPriv))
 
-static void           yelp_settings_class_init   (YelpSettingsClass    *klass);
-static void           yelp_settings_init         (YelpSettings         *settings);
 static void           yelp_settings_constructed  (GObject              *object);
-static void           yelp_settings_dispose      (GObject              *object);
 static void           yelp_settings_finalize     (GObject              *object);
 static void           yelp_settings_get_property (GObject              *object,
 						  guint                 prop_id,
@@ -119,7 +116,6 @@ yelp_settings_class_init (YelpSettingsClass *klass)
     gint i;
 
     object_class->constructed  = yelp_settings_constructed;
-    object_class->dispose  = yelp_settings_dispose;
     object_class->finalize = yelp_settings_finalize;
     object_class->get_property = yelp_settings_get_property;
     object_class->set_property = yelp_settings_set_property;
@@ -244,10 +240,10 @@ yelp_settings_constructed (GObject *object)
     YelpSettings *settings = YELP_SETTINGS (object);
     GDBusConnection *connection;
     GVariant *ret, *names;
-    GVariant *ret2;
     GVariantIter iter;
     gchar *name;
     gboolean env_shell, env_classic, env_panel, env_unity, env_xfce;
+    gchar *os_release = NULL;
     GError *error = NULL;
 
     connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
@@ -329,17 +325,68 @@ yelp_settings_constructed (GObject *object)
     else if (env_panel)
         yelp_settings_set_if_token (settings, "platform:gnome-panel");
 
+    g_file_get_contents ("/etc/os-release", &os_release, NULL, NULL);
+    if (os_release == NULL)
+        g_file_get_contents ("/usr/lib/os-release", &os_release, NULL, NULL);
+
+    if (os_release != NULL) {
+        gint i;
+        gchar **lines = g_strsplit(os_release, "\n", -1);
+        gchar *osid = NULL, *osversion = NULL, *end;
+        g_free (os_release);
+
+        for (i = 0; lines[i] != NULL; i++) {
+            if (g_str_has_prefix (lines[i], "ID=")) {
+                if (lines[i][3] == '"') {
+                    end = strchr (lines[i] + 4, '"');
+                    if (end != NULL)
+                        osid = g_strndup (lines[i] + 4, end - lines[i] - 4);
+                }
+                else if (lines[i][3] == '\'') {
+                    end = strchr (lines[i] + 4, '\'');
+                    if (end != NULL)
+                        osid = g_strndup (lines[i] + 4, end - lines[i] - 4);
+                }
+                else {
+                    osid = g_strdup (lines[i] + 3);
+                }
+            }
+            else if (g_str_has_prefix (lines[i], "VERSION_ID=")) {
+                if (lines[i][11] == '"') {
+                    end = strchr (lines[i] + 12, '"');
+                    if (end != NULL)
+                        osversion = g_strndup (lines[i] + 12, end - lines[i] - 12);
+                }
+                else if (lines[i][11] == '\'') {
+                    end = strchr (lines[i] + 12, '\'');
+                    if (end != NULL)
+                        osversion = g_strndup (lines[i] + 12, end - lines[i] - 12);
+                }
+                else {
+                    osversion = g_strdup (lines[i] + 11);
+                }
+            }
+        }
+
+        if (osid) {
+            gchar *token = g_strconcat("platform:", osid, NULL);
+            yelp_settings_set_if_token (settings, token);
+            g_free (token);
+            if (osversion) {
+                token = g_strconcat("platform:", osid, "-", osversion, NULL);
+                yelp_settings_set_if_token (settings, token);
+                g_free (token);
+                g_free (osversion);
+            }
+            g_free (osid);
+        }
+
+        g_strfreev(lines);
+    }
+
     yelp_settings_set_if_token (settings, "action:install");
 }
 
-static void
-yelp_settings_dispose (GObject *object)
-{
-    YelpSettings *settings = YELP_SETTINGS (object);
-
-    G_OBJECT_CLASS (yelp_settings_parent_class)->dispose (object);
-}
-
 static void
 yelp_settings_finalize (GObject *object)
 {
@@ -862,7 +909,7 @@ yelp_settings_get_all_params (YelpSettings *settings,
         g_string_append (malstr, (gchar *) envi->data);
         if (g_str_has_prefix ((gchar *) envi->data, "platform:")) {
             g_string_append_c (dbstr, ';');
-            g_string_append (dbstr, (gchar *) (envi->data + 9));
+            g_string_append (dbstr, (gchar *) (envi->data) + 9);
         }
     }
     g_string_append_c (malstr, '\'');
@@ -917,21 +964,21 @@ gtk_theme_changed (GtkSettings  *gtk_settings,
 
     /* YELP_SETTINGS_COLOR_BASE */
     g_snprintf (settings->priv->colors[YELP_SETTINGS_COLOR_BASE], 8, "#%02X%02X%02X",
-                (gint) (base.red * 255), (gint) (base.green * 255), (gint) (base.blue * 255));
+                (guint) (base.red * 255), (guint) (base.green * 255), (guint) (base.blue * 255));
 
     /* YELP_SETTINGS_COLOR_TEXT */
     g_snprintf (settings->priv->colors[YELP_SETTINGS_COLOR_TEXT], 8, "#%02X%02X%02X",
-                (gint) (text.red * 255), (gint) (text.green * 255), (gint) (text.blue * 255));
+                (guint) (text.red * 255), (guint) (text.green * 255), (guint) (text.blue * 255));
 
     /* YELP_SETTINGS_COLOR_LINK */
     gtk_style_context_get_color (linkcontext, GTK_STATE_FLAG_LINK, &link);
     g_snprintf (settings->priv->colors[YELP_SETTINGS_COLOR_LINK], 8, "#%02X%02X%02X",
-                (gint) (link.red * 255), (gint) (link.green * 255), (gint) (link.blue * 255));
+                (guint) (link.red * 255), (guint) (link.green * 255), (guint) (link.blue * 255));
 
     /* YELP_SETTINGS_COLOR_LINK_VISITED */
     gtk_style_context_get_color (linkcontext, GTK_STATE_FLAG_VISITED, &link);
     g_snprintf (settings->priv->colors[YELP_SETTINGS_COLOR_LINK_VISITED], 8, "#%02X%02X%02X",
-                (gint) (link.red * 255), (gint) (link.green * 255), (gint) (link.blue * 255));
+                (guint) (link.red * 255), (guint) (link.green * 255), (guint) (link.blue * 255));
 
     /* YELP_SETTINGS_COLOR_TEXT_LIGHT */
     hsv_to_hex (text_h, text_s, text_v - ((text_v - base_v) * 0.25),
@@ -1113,12 +1160,10 @@ static void
 hsv_to_hex (gdouble h, gdouble s, gdouble v, gchar *str)
 {
     gint hue;
-    gdouble c;
     gdouble m1, m2, m3;
     gdouble r, g, b;
-    guint8 red, green, blue;
+    guint red, green, blue;
 
-    c = v * s;
     h /= 60;
     hue = (int) h;
     m1 = v * (1 - s);
@@ -1139,6 +1184,8 @@ hsv_to_hex (gdouble h, gdouble s, gdouble v, gchar *str)
         g = m1; r = m3; break;
     case 5:
         g = m1; b = m2; break;
+    default:
+        g_assert_not_reached (); break;
     }
 
     red = r * 255;
